package be.iminds.ilabt.jfed.experimenter_gui.editor.properties.map;

import be.iminds.ilabt.jfed.experimenter_gui.config.JFedGuiConfig;
import be.iminds.ilabt.jfed.experimenter_gui.util.StringConverters;
import de.saring.leafletmap.*;
import javafx.application.Platform;
import javafx.collections.FXCollections;
import javafx.collections.ObservableList;
import javafx.collections.transformation.FilteredList;
import javafx.event.Event;
import javafx.event.EventHandler;
import javafx.fxml.FXML;
import javafx.scene.control.Button;
import javafx.scene.control.ComboBox;
import javafx.scene.layout.BorderPane;
import javafx.stage.Stage;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.*;

/**
 * User: twalcari
 * Date: 1/8/2015
 * Time: 14:34
 */
public class NodesMapDialog {
    private static final Logger LOG = LoggerFactory.getLogger(NodesMapDialog.class);


    @FXML
    private BorderPane root;

    private LeafletMapView mapView;

    @FXML
    private ComboBox<JFedGuiConfig.ComponentInfo> nodesComboBox;

    @FXML
    private Button selectNodeButton;

    private boolean mapViewIsInitialized = false;

//    private ClusteredGoogleMap map;

    private ObservableList<JFedGuiConfig.ComponentInfo> nodes = null;
    private final Map<JFedGuiConfig.ComponentInfo, LatLong> nodeLocations = new HashMap<>();

    private FilteredList<JFedGuiConfig.ComponentInfo> nodesWithinBounds;

    private JFedGuiConfig.ComponentInfo result = null;
//    private Marker selectedLocationMarker;


    @FXML
    private void initialize() {
        mapView = new LeafletMapView();

        root.setCenter(mapView);

        initializeMap();

        nodesComboBox.setConverter(StringConverters.COMPONENT_INFO_CONVERTER);
        selectNodeButton.disableProperty().bind(nodesComboBox.valueProperty().isNull());

        nodesComboBox.setOnAction(event -> {
            JFedGuiConfig.ComponentInfo node = nodesComboBox.getValue();
            if (node != null && nodeLocations.get(node) != null) {
                mapView.panTo(nodeLocations.get(node));
                mapView.setZoom(18);
            }
        });

    }

    private void initializeMap() {
        MapConfig mapConfig = new MapConfig(
                Collections.singletonList(MapLayer.MAPBOX),
                new ZoomControlConfig(true, ControlPosition.BOTTOM_LEFT),
                new ScaleControlConfig(true, ControlPosition.BOTTOM_RIGHT, true)
        );


//        mapOptions.mapType(MapTypeIdEnum.ROADMAP)
//                .center(new LatLong(41.711, 26.828))
//                .overviewMapControl(true)
//                .panControl(true)
//                .rotateControl(true)
//                .scaleControl(true)
//                .streetViewControl(true)
//                .zoomControl(true)
//                .zoom(3)
//                .maxZoom(20); //was 18


        mapView.displayMap(mapConfig)
                .whenCompleteAsync((state, throwable) -> {
                    mapViewIsInitialized = true;
                    tryInitializeMap();
                }, Platform::runLater);

    }


    private void tryInitializeMap() {
        if (nodes == null) return;
        if (!mapViewIsInitialized) return;

        LOG.debug("Initialising nodesMap for " + nodes.size() + " nodes on " + nodeLocations.size() + " locations.");

        List<Marker> markerList = new ArrayList<>();

        nodes.stream().filter(JFedGuiConfig.ComponentInfo::hasLocation).forEach(
                node ->
                {
                    try {
                        String title = node.getUrn().getEncodedResourceName();

                        LatLong position = new LatLong(Double.parseDouble(node.getLocation().getLatitude()),
                                Double.parseDouble(node.getLocation().getLongitude()));

                        nodeLocations.put(node, position);

                        Marker mcm = new Marker(position,
                                title, ColorMarkerIcon.BLUE_MARKER, 0);
                        mcm.setOnClickEventHandler(event -> selectNode(node));
                        markerList.add(mcm);

                        LOG.trace("Adding marker {} at location {}", title, node.getLocation());
                    } catch (Exception ex) {
                        LOG.error("Could not add node {} to map", node.getUrn());
                    }
                });

        MarkerClusterGroup mcg = new MarkerClusterGroup(markerList);
        mapView.addMarkerClusterGroup(mcg);
    }

    private void clearNodeSelection() {
        nodesComboBox.setValue(null);
    }

    private void selectNode(JFedGuiConfig.ComponentInfo node) {
        assert (node.getLocation() != null);
        mapView.panTo(nodeLocations.get(node));

        nodesComboBox.setValue(node);
    }

    public void setNodes(List<JFedGuiConfig.ComponentInfo> nodes) {
        if (this.nodes != null)
            throw new IllegalStateException("Already initialized");

        this.nodes = FXCollections.observableList(nodes);
        this.nodesWithinBounds = new FilteredList<>(this.nodes);

        nodesComboBox.setItems(nodesWithinBounds);

        tryInitializeMap();
    }

    @FXML
    private void selectNode() {
        result = nodesComboBox.getValue();
        ((Stage) selectNodeButton.getScene().getWindow()).close();
    }

    public JFedGuiConfig.ComponentInfo getResult() {
        return result;
    }
}
